using NUnit.Framework;
using NSubstitute;
using MobileChargingStation.Lib.Controller;
using MobileChargingStation.Lib.Models;
using UsbSimulator;
using System.ComponentModel;

namespace MobileChargingStation.Lib.Tests
{
    [TestFixture]
    public class ChargeControlTests
    {
        private ChargeControl _uut;
        private IUsbCharger _usbCharger;
        private Display _display;

        [SetUp]
        public void Setup()
        {
            _usbCharger = Substitute.For<IUsbCharger>();
            _display = Substitute.For<Display>();

            _uut = new ChargeControl(_usbCharger, _display);
        }

        /*******************************************************
        *               Equivalence Partitioning (EP)          *
        *******************************************************/

        // først tester vi usb charger connected og disconnected 
        // ep da det er 2 tilstande true eller false, dette tester vi her

        [Test]
        public void StartCharging_DeviceNotConnected_DisplaysErrorMessage()
        {
            // Arrange
            _usbCharger.Connected.Returns(false);

            // Act
            _uut.StartCharging();

            // Assert
            _display.Received(1).ShowMessage("No device connected, cannot start charging.");
            _usbCharger.DidNotReceive().StartCharge();
        }

        [Test]
        public void StartCharging_DeviceConnected_DisplaysStartMessageAndStartsCharging()
        {
            // Arrange
            _usbCharger.Connected.Returns(true);

            // Act
            _uut.StartCharging();

            // Assert
            _display.Received(1).ShowMessage("Starting USB charging...");
            _usbCharger.Received(1).StartCharge();
        }

        // herunder tester vi, at vi kan stoppe opladningen, og at opladningen faktisk stopper
        // Vi tester også, at der vises en korrekt besked på displayet, når opladningen stoppes.
        // Derudover sikrer vi, at metoden ikke kaster en fejl, hvis opladningen allerede er stoppet.

        [Test]
        public void StopCharging_DisplaysStopMessageAndStopsCharging()
        {
            
            _uut.StopCharging();

            
            _display.Received(1).ShowMessage("Stopping USB charging...");
            _usbCharger.Received(1).StopCharge();
        }

        [Test]
        public void StopCharging_WhenNotCharging_DoesNotThrowException()
        {
            
            Assert.DoesNotThrow(() => _uut.StopCharging());
        }

        /*******************************************************
        *               Boundary Value Analysis (BVA)          *
        *******************************************************/

        // bva herunder test vi at vi tester grænseværdierne for USB-strømmen.
        // vi kører test på 500, 0 og -100, som er grænseværdierne for strømmen.
        // Vi tester også, at der vises en korrekt besked på displayet for hver af disse værdier.

        [Test]
        public void HandleCurrentValueChanged_DisplaysCurrentValue()
        {
            var eventArgs = new CurrentEventArgs { Current = 500 };

            _usbCharger.CurrentValueEvent += Raise.EventWith(eventArgs);

            _display.Received(1).ShowMessage("USB Current changed: 500 mA");
        }

        [Test]
        public void HandleCurrentValueChanged_CurrentIsZero_DisplaysNoCurrentMessage()
        {
            var eventArgs = new CurrentEventArgs { Current = 0 };

            _usbCharger.CurrentValueEvent += Raise.EventWith(eventArgs);

            
            _display.Received(1).ShowMessage("No current detected.");
        }

        [Test]
        public void HandleCurrentValueChanged_CurrentIsNegative_DisplaysErrorMessage()
        {
            
            var eventArgs = new CurrentEventArgs { Current = -100 };

            
            _usbCharger.CurrentValueEvent += Raise.EventWith(eventArgs);

            
            _display.Received(1).ShowMessage("Error: Negative current detected.");
        }

        [Test]
        public void HandleCurrentValueChanged_CurrentIsMaximum_DisplaysCurrentValue()
        {
            
            var eventArgs = new CurrentEventArgs { Current = int.MaxValue };

            
            _usbCharger.CurrentValueEvent += Raise.EventWith(eventArgs);

        
            _display.Received(1).ShowMessage($"USB Current changed: {int.MaxValue} mA");
        }

        /*******************************************************
        *                   Zombie Testing                     *
        *******************************************************/

        //********************************************************          

       
    
        // vi tester i følgende edgecases 
        // Når opladning forsøges startet uden en tilsluttet enhed.
        // Når opladning stoppes, selvom opladningen allerede er stoppet.
        // Når hændelsen CurrentValueEvent udløses, men der ikke er nogen subscribers.

        [Test]
        public void StartCharging_DeviceNotConnected_DoesNotStartCharging()
        {
            
            _usbCharger.Connected.Returns(false);

            
            _uut.StartCharging();

            
            _usbCharger.DidNotReceive().StartCharge();
        }

        [Test]
        public void StopCharging_WhenAlreadyStopped_DoesNotThrowException()
        {
            
            Assert.DoesNotThrow(() => _uut.StopCharging());
        }

        [Test]
        public void HandleCurrentValueChanged_WhenNoSubscribers_DoesNotThrowException()
        {
            
            var eventArgs = new CurrentEventArgs { Current = 500 };

            
            Assert.DoesNotThrow(() => _usbCharger.CurrentValueEvent += Raise.EventWith(eventArgs));
        }
    }
}