using NUnit.Framework;
using NSubstitute;
using MobileChargingStation.Lib.Controller;
using MobileChargingStation.Lib.Models;
using MobileChargingStation.Lib.Utils;
using UsbSimulator;

namespace MobileChargingStation.Lib.Tests
{
    // opsætning af NSubstitute, og test af StationControl
    // herunder opstilles stationcontrol, med substitutter for door, usbcharger og rfidreader
    [TestFixture]
    public class StationControlTests
    {
        private StationControl _uut;
        private IDoor _door;
        private IUsbCharger _usbCharger;
        private IRfidReader _rfidReader;

        private Display _display;
        private ILogger _logger;

        [SetUp]
        public void Setup()
        {
            _door = Substitute.For<IDoor>();
            _usbCharger = Substitute.For<IUsbCharger>();
            _rfidReader = Substitute.For<IRfidReader>();
            _display = Substitute.For<Display>();
            _logger = Substitute.For<ILogger>(); 

            _uut = new StationControl(_door, _usbCharger, _rfidReader, _display, _logger);
        }

        /*******************************************************
        *               Equivalence Partitioning (EP)          *
        *******************************************************/
        // Herunder tester vi EP, vha. følgende scenarier:


        //Valid RFID + USB connected, ladning starter.
        //Valid RFID + USB not connected: ladning starter ikke.
        //Invalid RFID + USB connected: ladning startes ikke.

        

        // case (true, true, true, false):
        // forventet er at vi vil kalde function lockdoor, 
        // som vil resultere i at (false, false, false, false);
        // Vi vil skrive sådanne kommentarer til cases, da det er svært at lave navne til dem 

        [Test]
        public void DoorChangedEvent_startOpladning()
        {
            // Arrange
            _uut.SetRfidId(123);
            _rfidReader.DetectRfid(123).Returns(true);
            _usbCharger.Connected.Returns(true);

            
            _door.DoorChangedEvent += Raise.EventWith(new DoorChangedEventArgs
            {
                DoorOpen = false,
                DoorUnlocked = false,
                DoorAvailable = false,
                recharging = false
            });

           
            _usbCharger.Received(1).StartCharge();
            _door.Received(1).SetDoorStatus(false, false, false, true);
        }

        

        // case (true, true, true, false):
        // forventet er at den ikke vil kalde function lockdoor, 
        // da vi har sat connected til false, så den vil ikke starte med at oplade
        // Vi vil skrive sådanne kommentarer til cases, da det er svært at lave navne til dem 

        [Test]
        public void DoorChangedEvent_startOpladning_Not_Connected()
        {
            
            _usbCharger.Connected.Returns(false);

          
            _door.DoorChangedEvent += Raise.EventWith(new DoorChangedEventArgs
            {
                DoorOpen = true,
                DoorUnlocked = true,
                DoorAvailable = true,
                recharging = false
            });

           
            _door.DidNotReceive().SetDoorStatus(false, false, false, false);
        }

       

        // case (false, false, false):
        // vi forventer ikke at charging door vil blive kaldt, da rfid er forkert
        // vi har hardcoded rfid id til 123, men sat den til at være 124, så vi kan teste om det virker
        // Vi forventer den vil ikke starte med at oplade

        [Test]
        public void DoorChangedEvent_Startcharge_wrongrfid()
        {
            
            _uut.SetRfidId(124);
            _rfidReader.DetectRfid(124).Returns(false);
            _usbCharger.Connected.Returns(true);

          
            _door.DoorChangedEvent += Raise.EventWith(new DoorChangedEventArgs
            {
                DoorOpen = false,
                DoorUnlocked = false,
                DoorAvailable = false,
                recharging = false
            });

            
            _usbCharger.DidNotReceive().StartCharge();
            _door.DidNotReceive().SetDoorStatus(false, false, false, true);
        }   

        

        /*******************************************************
        *               Boundary Value Analysis (BVA)          *
        *******************************************************/
        // Vi bruger bva når for at teste grænseværdierne for RFID-læseren.
        //  Når opladning er aktiv, og en gyldig RFID detekteres.
        //  Når opladning er aktiv, og en ugyldig RFID detekteres.


        // case (false, false, false, true):
        // vi forventer at vi vil kalde function unlockdoor, 
        // vi forventer at stopcharging door vil blive kaldt, da rfid er sat til true
        // staten skulle gerne blive (false, true, true, false);
        // vi har hardcoded rfid id til 123, sat den til at være 123, så vi kan teste om det virker
        // Vi forventer den vil kalde stopcharge og få en ny status

          [Test]
        public void DoorChangedEvent_DoorClosedCharging_RfidDetected_ShouldStopChargingAndUnlockDoor()
        {
            
            _uut.SetRfidId(123);
            _rfidReader.DetectRfid(123).Returns(true);

            
            _door.DoorChangedEvent += Raise.EventWith(new DoorChangedEventArgs
            {
                DoorOpen = false,
                DoorUnlocked = false,
                DoorAvailable = false,
                recharging = true
            });

            
            _usbCharger.Received(1).StopCharge();
            _door.Received(1).SetDoorStatus(false, true, true, false);
        }

        [Test]
        public void DoorChangedEvent_DoorClosedCharging_InvalidRfid_ShouldNotStopCharging()
        {
            
            _uut.SetRfidId(123);
            _rfidReader.DetectRfid(124).Returns(false);

            
            _door.DoorChangedEvent += Raise.EventWith(new DoorChangedEventArgs
            {
                DoorOpen = false,
                DoorUnlocked = false,
                DoorAvailable = false,
                recharging = true
            });

           
            _usbCharger.DidNotReceive().StopCharge();
            _door.DidNotReceive().SetDoorStatus(false, true, true, false);
        }
        //##################################################
       [Test]
        public void Case_FourFalse_RfidDetected_UsbConnected_ShouldStartCharging()
        {
            
            _uut.SetRfidId(123);
            _rfidReader.DetectRfid(123).Returns(true);
            _usbCharger.Connected.Returns(true);

            
            _door.DoorChangedEvent += Raise.EventWith(new DoorChangedEventArgs
            {
                DoorOpen = false,
                DoorUnlocked = false,
                DoorAvailable = false,
                recharging = false
            });

            
            _usbCharger.Received(1).StartCharge();
            _door.Received(1).SetDoorStatus(false, false, false, true);
      }

      [Test]
        public void Case_FourFalse_RfidDetected_UsbNotConnected_ShouldShowError()
        {
            
            _uut.SetRfidId(123);
            _rfidReader.DetectRfid(123).Returns(true);
            _usbCharger.Connected.Returns(false);

            
            _door.DoorChangedEvent += Raise.EventWith(new DoorChangedEventArgs
            {
                DoorOpen = false,
                DoorUnlocked = false,
                DoorAvailable = false,
                recharging = false
            });

            
            _display.Received(1).ShowMessage("Usb ikke tilsluttet... døren åbnes op");
        }

       




        /*******************************************************
        *                   Zombie Testing                     *
        *******************************************************/

        
        
        // Når døren er i standardtilstand (lukket, ulåst, og tilgængelig), 
        // og der ikke sker nogen handling, sikrer vi, at systemet ikke udfører unødvendige operationer.
        // 
        // Når døren er lukket og ulåst, og en gyldig RFID detekteres, 
        // sikrer vi, at døren låses korrekt.
        // 
        // Når døren er lukket og ulåst, og en ugyldig RFID detekteres, 
        // sikrer vi, at døren ikke låses, og ingen unødvendige handlinger udføres.




        // case (false, true, true, false):
        // tester bare at hvis døren er default, så vil der ikke ske noget
        // did not receive betyder at den ikke modtager event til at låse døren

        [Test]
        public void test_af_default_skabbet()
        {
            
            _uut.SetRfidId(124);
            _rfidReader.DetectRfid(124).Returns(true);

            
            _door.DoorChangedEvent += Raise.EventWith(new DoorChangedEventArgs
            {
                DoorOpen = false,
                DoorUnlocked = true,
                DoorAvailable = true,
                recharging = false
            });

            
            _usbCharger.DidNotReceive().StartCharge();
            _door.DidNotReceive().SetDoorStatus(false, false, false, false);
        }

        [Test]
        public void DoorChangedEvent_DoorClosedUnlocked_RfidDetected_ShouldLockDoor()
        {
           
            _uut.SetRfidId(123);
            _rfidReader.DetectRfid(123).Returns(true);

           
            _door.DoorChangedEvent += Raise.EventWith(new DoorChangedEventArgs
            {
                DoorOpen = false,
                DoorUnlocked = true,
                DoorAvailable = false,
                recharging = false
            });

            
            _door.Received(1).SetDoorStatus(false, false, false, false);
        }

        [Test]
        public void DoorChangedEvent_DoorClosedUnlocked_RfidNotDetected_ShouldNotLockDoor()
        {
            
            _uut.SetRfidId(123);
            _rfidReader.DetectRfid(123).Returns(false);

            
            _door.DoorChangedEvent += Raise.EventWith(new DoorChangedEventArgs
            {
                DoorOpen = false,
                DoorUnlocked = true,
                DoorAvailable = false,
                recharging = false
            });

            
            _door.DidNotReceive().SetDoorStatus(false, false, false, false);
        }

        [Test]
        public void DoorChangedEvent_DoorOpenUnlockedUnavailable_ShouldShowConnectMessage()
        {
           
            _door.DoorChangedEvent += Raise.EventWith(new DoorChangedEventArgs
            {
                DoorOpen = true,
                DoorUnlocked = false,
                DoorAvailable = false,
                recharging = false
            });

            
            _display.Received(1).ShowMessage("tislut svinet");

        }

 

        /*******************************************************
        *                                                     *
        *                                                     *
        *        Test funktioner til at ændre doorstatus      *
        *                                                     *
        *******************************************************/

        // herunder tester vi de forskellige funktioner til at ændre døren status
        

        [Test]
        public void OpenDoorTest()
        {
            
            _uut.OpenDoor();

           
            _door.Received(1).SetDoorStatus(true, false, false, false);
        }

        [Test]
        public void UnlockDoorTest()
        {
            
            _uut.UnlockDoor();

            
            _door.Received(1).SetDoorStatus(false, true, true, false);
        }

        [Test]
        public void LockDoorTest()
        {
            
            _uut.LockDoor();

            
            _door.Received(1).SetDoorStatus(false, false, false, false);
        }

        [Test]
        public void ChargingDoorTest()
        {
            
            _uut.ChargingLockedDoor();

            
            _door.Received(1).SetDoorStatus(false, false, false, true);
        }
       

        [Test]
        public void CloseDoorTest()
        {
            
            _uut.CloseDoor();

            
            _door.Received(1).SetDoorStatus(false, true, false, false);
        }

        [Test]
        public void DetectRfid_Called_RfidReaderDetectCalled()
        {
            
            int testRfid = 789;

            
            _uut.DetectRfid(testRfid);

            
            _rfidReader.Received(1).DetectRfid(testRfid);
        }

        [Test]
        public void SetRfidId_SetsCorrectId_UsedInSubsequentOperations()
        {
            
            int testId = 123;

            
            _uut.SetRfidId(testId);
            _uut.DetectRfid(testId);

            
            _rfidReader.Received(1).DetectRfid(testId);
        }
 
        

    }
    
}