using System;
using System.IO;
using NUnit.Framework;
using MobileChargingStation.Lib.Models;

namespace MobileChargingStation.Lib.Tests
{
    [TestFixture]
    public class DisplayTests
    {
        private Display _uut;
        private StringWriter _output;
        private TextWriter _originalConsole;

        [SetUp]
        public void Setup()
        {
            _uut = new Display();
            _originalConsole = Console.Out;
            _output = new StringWriter();
            Console.SetOut(_output);
        }

        [TearDown]
        public void Cleanup()
        {
            Console.SetOut(_originalConsole);
            _output.Dispose();
        }

        /*******************************************************
        *               Equivalence Partitioning (EP)          *
        *******************************************************/

        /*******************************************************



        // - vi tester for at når vi skriver noget til display så skal det være DISPLAY: + det vi skriver
        //  derudover tester vi en tom strn, så det kun er DISPLAY: der kommer
        //  og til sidst tester vi en med mellem  så det også kun er DISPLAY: der kommer
        // derudover tester vi også en med special characters, så det også virker

        */

        [Test]
        public void ShowMessage_AddsDisplayPrefix()
        {
            
            _uut.ShowMessage("Hello");

            
            Assert.That(_output.ToString().Trim(), Is.EqualTo("DISPLAY: Hello"));
        }

        [Test]
        public void ShowMessage_WithEmptyInput_ShowsOnlyPrefix()
        {
            
            _uut.ShowMessage("");

            
            Assert.That(_output.ToString().Trim(), Is.EqualTo("DISPLAY:"));
        }

        [Test]
        public void ShowMessage_WithWhitespaceInput_ShowsPrefixAndWhitespace()
        {
            
            _uut.ShowMessage("   ");

            
            Assert.That(_output.ToString().Trim(), Is.EqualTo("DISPLAY:"));
        }

        [Test]
        public void ShowMessage_WithSpecialCharacters_DisplaysCorrectly()
        {
            
            _uut.ShowMessage("@#$%^&*()");

            
            Assert.That(_output.ToString().Trim(), Is.EqualTo("DISPLAY: @#$%^&*()"));
        }

        /*******************************************************
        *               Boundary Value Analysis (BVA)          *
        *******************************************************/

        // Bva tester vi ved først at have strenge med 1 og 1000 tegn for at teste grænserne

        [Test]
        public void ShowMessage_WithMinimumLengthMessage_DisplaysCorrectly()
        {
            
            _uut.ShowMessage("A");

            
            Assert.That(_output.ToString().Trim(), Is.EqualTo("DISPLAY: A"));
        }

        [Test]
        public void ShowMessage_WithMaximumLengthMessage_DisplaysCorrectly()
        {
            
            string maxLengthMessage = new string('A', 1000); 

            
            _uut.ShowMessage(maxLengthMessage);

            
            Assert.That(_output.ToString().Trim(), Is.EqualTo($"DISPLAY: {maxLengthMessage}"));
        }

       

        /*******************************************************
        *                   Zombie Testing                     *
        *******************************************************/

        // Zombie testing tester vi ved at have en null string og se om der kommer en exception
        // og at ved flere kald til metoden så kommer der flere beskeder i rigtig rækkefølge
        // og til sidst tester vi at der ikke kommer en fejl hvis vi kalder beskeden 2 gange og den ene besked er tom

        [Test]
        public void ShowMessage_NullInput_DoesNotThrowException()
        {
            
            Assert.DoesNotThrow(() => _uut.ShowMessage(null));
        }

        [Test]
        public void ShowMessage_MultipleCalls_DisplaysAllMessages()
        {
            
            _uut.ShowMessage("First Message");
            _uut.ShowMessage("Second Message");

            
            string[] lines = _output.ToString().Trim().Split(Environment.NewLine);
            Assert.That(lines[0], Is.EqualTo("DISPLAY: First Message"));
            Assert.That(lines[1], Is.EqualTo("DISPLAY: Second Message"));
        }

        [Test]
        public void ShowMessage_EmptyMessageAfterValidMessage_DisplaysBothCorrectly()
        {
            
            _uut.ShowMessage("Valid Message");
            _uut.ShowMessage("");

            
            string[] lines = _output.ToString().Trim().Split(Environment.NewLine);
            Assert.That(lines[0], Is.EqualTo("DISPLAY: Valid Message"));
            Assert.That(lines[1], Is.EqualTo("DISPLAY:"));
        }
    }
}
