using NUnit.Framework;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using NSubstitute;
using MobileChargingStation.Lib.Models;

namespace MobileChargingStation.Lib.Tests
{
    public class DoorUnitTest
    {
        private Door? _uut;

        [SetUp]
        public void Setup()
        {
            _uut = new Door();
        }

        /*******************************************************
        *               Equivalence Partitioning (EP)          *
        *******************************************************/
        // her tester vi at status ændres, når event raises og at den ændres korrekt
        // vi tester også at den ikke ændres når der ikke er nogen event
        

        [Test]
        public void SetDoorStatus_ChangesState_RaisesDoorChangedEvent()
        {
            // Arrange
            bool eventRaised = false;
            DoorChangedEventArgs? eventArgs = null;

            _uut!.DoorChangedEvent += (sender, args) =>
            {
                eventRaised = true;
                eventArgs = args;
            };

            
            _uut.SetDoorStatus(true, false, true, false);

            
            Assert.That(eventRaised, Is.True);
            Assert.That(eventArgs, Is.Not.Null);
            Assert.That(eventArgs!.DoorOpen, Is.True);
            Assert.That(eventArgs.DoorUnlocked, Is.False);
            Assert.That(eventArgs.DoorAvailable, Is.True);
            Assert.That(eventArgs.recharging, Is.False);
        }

        [Test]
        public void SetDoorStatus_UpdatesStateCorrectly()
        {
            
            bool eventRaised = false;

            _uut!.DoorChangedEvent += (sender, args) =>
            {
                eventRaised = true;
                Assert.That(args.DoorOpen, Is.True);
                Assert.That(args.DoorUnlocked, Is.True);
                Assert.That(args.DoorAvailable, Is.False);
                Assert.That(args.recharging, Is.True);
            };

            
            _uut.SetDoorStatus(true, true, false, true);

            
            Assert.That(eventRaised, Is.True);
        }

        /*******************************************************
        *               Boundary Value Analysis (BVA)          *
        *******************************************************/

        // - Ingen statusændring: Når inputværdierne er de samme som de forhenværende værdier, 
        //   skal der ikke udløses nogen hændelse.
        

        [Test]
        public void SetDoorStatus_NoStateChange_DoesNotRaiseEvent()
        {
            
            _uut!.SetDoorStatus(false, false, true, false);
            bool eventRaised = false;

            _uut.DoorChangedEvent += (sender, args) =>
            {
                eventRaised = true;
            };

            
            _uut.SetDoorStatus(false, false, true, false);

            
            Assert.That(eventRaised, Is.False);
        }

        /*******************************************************
        *                   Zombie Testing                     *
        *******************************************************/

        [Test]
        public void SetDoorStatus_NullEventHandler_DoesNotThrowException()
        {
            
            Assert.DoesNotThrow(() => _uut!.SetDoorStatus(true, false, true, false));
        }

        [Test]
        public void SetDoorStatus_MultipleStateChanges_RaisesEventEachTime()
        {
            
            int eventCount = 0;

            _uut!.DoorChangedEvent += (sender, args) =>
            {
                eventCount++;
            };

            _uut.SetDoorStatus(true, false, true, false);
            _uut.SetDoorStatus(false, true, false, true);

            Assert.That(eventCount, Is.EqualTo(2));
        }
    }
}
