using NUnit.Framework;
using System;
using MobileChargingStation.Lib.Models;

namespace MobileChargingStation.Lib.Tests
{
    public class RfidReaderUnitTest
    {
        private RfidReader? _uut;

        [SetUp]
        public void Setup()
        {
            _uut = new RfidReader();
        }

        /*******************************************************
        *               Equivalence Partitioning (EP)          *
        *******************************************************/
        // Vi tester Ep ved at sikre følgnde 
        //  Gyldige RFID'er Sikrer, at metoden returnerer true og udløser RfidChangedEvent med de korrekte argumenter.
        //  Ugyldige RFID'er: Sikrer, at metoden returnerer false og ikke udløser RfidChangedEvent.
        // 
        

        [Test]
        public void DetectRfid_ValidId_ReturnsTrue_AndRaisesEvent()
        {
           
            bool eventRaised = false;
            RfidDetectedChangedEventArgs? eventArgs = null;
            int validRfidID = 123; 

            _uut!.RfidChangedEvent += (sender, args) =>
            {
                eventRaised = true;
                eventArgs = args;
            };

           
            bool result = _uut.DetectRfid(validRfidID);

            
            Assert.That(result, Is.True);
            Assert.That(eventRaised, Is.True);
            Assert.That(eventArgs, Is.Not.Null);
            Assert.That(eventArgs!.RfidID, Is.EqualTo(validRfidID));
        }

        [Test]
        public void DetectRfid_InvalidId_ReturnsFalse_AndDoesNotRaiseEvent()
        {
           
            bool eventRaised = false;
            int invalidRfidID = 456; 

            _uut!.RfidChangedEvent += (sender, args) =>
            {
                eventRaised = true;
            };

            bool result = _uut.DetectRfid(invalidRfidID);

           
            Assert.That(result, Is.False);
            Assert.That(eventRaised, Is.False);
        }

        /*******************************************************
        *               Boundary Value Analysis (BVA)          *
        *******************************************************/
        // 
        //  Maksimale forsøg Sikrer, at metoden returnerer false, når det maksimale antal forsøg er nået.
        //  Gentagne ugyldige RFID'er: Validerer, at flere ugyldige forsøg ikke udløser hændelser eller ændrer adfærd.
        // 

       


        [Test]
        public void DetectRfid_MaxAttemptsReached_ReturnsFalse()
        {
            int invalidRfidID = 456;
            int maxAttempts = 3; 
            
        
            for (int i = 0; i < maxAttempts - 1; i++)
            {
                _uut!.DetectRfid(invalidRfidID);
            }
            
        
            bool result = _uut!.DetectRfid(invalidRfidID);
            
            Assert.That(result, Is.False);
        }

        [Test]
        public void DetectRfid_InvalidId_MultipleAttempts_StillReturnsFalse()
        {
            int invalidRfidID = 456;
            int maxAttempts = 3; 
            bool eventRaised = false;

            _uut!.RfidChangedEvent += (sender, args) =>
            {
                eventRaised = true;
            };
            
            
            for (int i = 0; i < maxAttempts + 2; i++)
            {
                _uut!.DetectRfid(invalidRfidID);
            }
            
            
            bool result = _uut!.DetectRfid(invalidRfidID);
            
            Assert.That(result, Is.False);
            Assert.That(eventRaised, Is.False);
        }

        /*******************************************************
        *                   Zombie Testing                     *
        *******************************************************/
     
        //  Ingen event handlers: Sikrer, at metoden ikke kaster undtagelser, når der ikke er nogen abonnenter på RfidChangedEvent.
        //  Altid fejlede RFID'er: Validerer, at metoden håndterer RFID'er, der altid fejler, uden problemer.
        //  Flere gyldige RFID'er: Sikrer, at metoden korrekt udløser hændelser for flere gyldige RFID'er i træk.
        // 
    

        [Test]
        public void DetectRfid_AlwaysFails_ReachesReturnFalseBranch()
        {
          
            int invalidRfidID = 999; 

           
            bool result = _uut!.DetectRfid(invalidRfidID);

            // Assert
            Assert.That(result, Is.False);
        }

        [Test]
        public void DetectRfid_NoEventHandlers_DoesNotThrowException()
        {
          
            int validRfidID = 123;

           
            Assert.DoesNotThrow(() => _uut!.DetectRfid(validRfidID));
        }

        [Test]
        public void DetectRfid_MultipleValidIds_RaisesEventEachTime()
        {
            int validRfidID1 = 123;
            int validRfidID2 = 124; 
            int eventCount = 0;

            _uut!.RfidChangedEvent += (sender, args) =>
            {
                eventCount++;
            };

            _uut.DetectRfid(validRfidID1);
            _uut.DetectRfid(validRfidID2);

            Assert.That(eventCount, Is.EqualTo(2));
        }

        
    }
}



        