using NUnit.Framework;
using System;
using System.IO;
using MobileChargingStation.Lib.Utils;
using NSubstitute;


namespace MobileChargingStation.Lib.Tests.UtilTest
{
    [TestFixture]
    public class FileLoggerTests
    {
        private string _testLogFilePath;
        private FileLogger? _fileLogger;

        [SetUp]
        public void Setup()
        {
            _testLogFilePath = Path.Combine(Path.GetTempPath(), "test_log.txt");
            _fileLogger = new FileLogger(_testLogFilePath);
        }

        [TearDown]
        public void Cleanup()
        {
            if (File.Exists(_testLogFilePath))
            {
                File.Delete(_testLogFilePath);
            }
        }

        /*******************************************************
        *                   Equivalence Partitioning           *
        *******************************************************/

        [Test]
        public void FileLogger_Log_ValidMessage_WritesToFile()
        {
            string testMessage = "This is a valid log message.";

            _fileLogger!.Log(testMessage);

            string fileContent = File.ReadAllText(_testLogFilePath);
            Assert.That(fileContent.Contains(testMessage), Is.True);
        }

        [Test]
        public void FileLogger_Log_EmptyMessage_WritesNothing()
        {
            string emptyMessage = "";

            _fileLogger!.Log(emptyMessage);

            string fileContent = File.ReadAllText(_testLogFilePath);
            Assert.That(fileContent.Contains(emptyMessage), Is.True);
        }

        [Test]
        public void FileLogger_Log_LargeMessage_WritesToFile()
        {
            string largeMessage = new string('A', 10000); 

            _fileLogger!.Log(largeMessage);

            
            string fileContent = File.ReadAllText(_testLogFilePath);
            Assert.That(fileContent.Contains(largeMessage), Is.True);
        }

        [Test]
public void Logger_Log_ValidMessage_CallsFileLogger()
{
    var mockFileLogger = Substitute.For<ILogger>();
    var logger = new Logger(mockFileLogger);
    string testMessage = "Test message";

    logger.Log(testMessage);

    mockFileLogger.Received(1).Log(testMessage);
}



        /*******************************************************
        *               Boundary Value Analysis                *
        *******************************************************/

        [Test]
        public void FileLogger_Log_MinimumLengthMessage_WritesToFile()
        {
            string minLengthMessage = "A"; 

            
            _fileLogger!.Log(minLengthMessage);

            string fileContent = File.ReadAllText(_testLogFilePath);
            Assert.That(fileContent.Contains(minLengthMessage), Is.True);
        }

        [Test]
        public void FileLogger_Log_MaximumLengthMessage_WritesToFile()
        {
            string maxLengthMessage = new string('B', 65535); 

            _fileLogger!.Log(maxLengthMessage);

            string fileContent = File.ReadAllText(_testLogFilePath);
            Assert.That(fileContent.Contains(maxLengthMessage), Is.True);
        }

        /*******************************************************
        *                   Zombie Testing                     *
        *******************************************************/

        

        [Test]
        public void FileLogger_Log_FileLocked_ThrowsIOException()
        {
            
            using (var fileStream = new FileStream(_testLogFilePath, FileMode.OpenOrCreate, FileAccess.Read, FileShare.None))
            {
                Assert.Throws<IOException>(() => _fileLogger!.Log("This should fail because the file is locked."));
            }
        }

        [Test]
        public void FileLogger_Log_AppendsMultipleMessages()
        {
            string message1 = "First log entry";
            string message2 = "Second log entry";

            _fileLogger!.Log(message1);
            _fileLogger.Log(message2);

            string fileContent = File.ReadAllText(_testLogFilePath);
            Assert.That(fileContent.Contains(message1), Is.True);
            Assert.That(fileContent.Contains(message2), Is.True);
        }

        [Test]
        public void FileLogger_Constructor_InvalidFilePath_ThrowsArgumentException()
        {
            string invalidPath = ""; 

            Assert.That(() => new FileLogger(invalidPath),
                Throws.InstanceOf<ArgumentException>().With.Message.Contains("The log file path cannot be null or empty."));
        }

        [Test]
        public void FileLogger_Constructor_WhitespaceFilePath_ThrowsArgumentException()
        {
            string whitespacePath = "   "; 

            Assert.That(() => new FileLogger(whitespacePath),
                Throws.InstanceOf<ArgumentException>().With.Message.Contains("The log file path cannot be null or empty."));
        }

        [Test]
public void FileLogger_Constructor_ValidFilePath_CreatesDirectory()
{
    string validPath = Path.Combine(Path.GetTempPath(), "ValidDirectory", "log.txt");

    var fileLogger = new FileLogger(validPath);

    Assert.That(Directory.Exists(Path.GetDirectoryName(validPath)), Is.True);

    Directory.Delete(Path.GetDirectoryName(validPath)!, true);
}

[Test]
public void FileLogger_Constructor_ExistingDirectory_DoesNotThrow()
{
    string existingDirectory = Path.Combine(Path.GetTempPath(), "ExistingDirectory");
    Directory.CreateDirectory(existingDirectory);
    string validPath = Path.Combine(existingDirectory, "log.txt");

    Assert.DoesNotThrow(() => new FileLogger(validPath));

    Directory.Delete(existingDirectory, true);
}

[Test]
public void FileLogger_Constructor_NestedDirectories_CreatesAllDirectories()
{
    string nestedPath = Path.Combine(Path.GetTempPath(), "ParentDirectory", "ChildDirectory", "log.txt");

    var fileLogger = new FileLogger(nestedPath);

    Assert.That(Directory.Exists(Path.GetDirectoryName(nestedPath)), Is.True);

    Directory.Delete(Path.Combine(Path.GetTempPath(), "ParentDirectory"), true);
}

[Test]
public void FileLogger_Constructor_ValidFilePath_DoesNotWriteToFile()
{
    string validPath = Path.Combine(Path.GetTempPath(), "log.txt");

    
    var fileLogger = new FileLogger(validPath);

    Assert.That(File.Exists(validPath), Is.False);

    if (File.Exists(validPath))
    {
        File.Delete(validPath);
    }
}

[Test]
public void FileLogger_Constructor_InvalidOrNotFullyQualifiedPath_ThrowsArgumentException()
{
    string invalidPath = "invalidPath\\log.txt"; 

    Assert.That(() => new FileLogger(invalidPath),
        Throws.InstanceOf<ArgumentException>().With.Message.Contains("The log file path is invalid or not fully qualified."));
}
[Test]
public void Logger_DefaultConstructor_InitializesFileLogger()
{
    
    Assert.DoesNotThrow(() => new Logger());
    
   
    var logger = new Logger();
    string testMessage = "Test message";
    
    
    Assert.DoesNotThrow(() => logger.Log(testMessage));
}

    }
}